package org.octopus.data.desensitized.core.strategy;

import org.apache.commons.lang3.StringUtils;
import org.octopus.data.desensitized.core.annotation.DesensitizedRule;

/**
 * @author lemon
 * @date 2021/8/27 11:14
 */
public abstract class AbstractDesensitizedStrategy<T> implements DesensitizedStrategy<T> {

    /**
     * 数据脱敏
     *
     * @param original
     * @return
     */
    @Override
    public final T convert(T original, DesensitizedRule desensitizedRule) {
        if (original == null) {
            return null;
        }

        if (!desensitizedRule.isEnable()) {
            return original;
        }

        DesensitizedRule defaultDesensitizedRule = this.obtainDefaultDesensitizedRule();

        if (desensitizedRule.getPrefixNoMaskLen() == DesensitizedRule.DEFAULT_VALUE) {
            desensitizedRule.setPrefixNoMaskLen(defaultDesensitizedRule.getPrefixNoMaskLen());
        }

        if (desensitizedRule.getSuffixNoMaskLen() == DesensitizedRule.DEFAULT_VALUE) {
            desensitizedRule.setSuffixNoMaskLen(defaultDesensitizedRule.getSuffixNoMaskLen());
        }

        if (StringUtils.isBlank(desensitizedRule.getMask())) {
            desensitizedRule.setMask(defaultDesensitizedRule.getMask());
        }

        return this.doConvert(original, desensitizedRule);
    }

    /**
     * 默认脱敏规则
     *
     * @return
     */
    abstract protected DesensitizedRule obtainDefaultDesensitizedRule();

    /**
     * 数据脱敏
     *
     * @param original
     * @param desensitizedRule
     * @return
     */
    abstract protected T doConvert(T original, DesensitizedRule desensitizedRule);
}
