package org.octopus.data.desensitized.core.util;

import org.apache.commons.lang3.StringUtils;
import org.octopus.data.desensitized.core.annotation.Desensitized;
import org.octopus.data.desensitized.core.annotation.DesensitizedRule;

/**
 * 数据脱敏工具类
 */
public class DesensitizedUtils {

    /**
     * @param prefixNoMaskLen
     * @param suffixNoMaskLen
     * @return
     */
    public static DesensitizedRule buildDefaultDesensitizedRule(int prefixNoMaskLen, int suffixNoMaskLen) {
        return DesensitizedUtils.buildDesensitizedRule(Boolean.TRUE, prefixNoMaskLen, suffixNoMaskLen, null);
    }

    /**
     * @param enable
     * @param prefixNoMaskLen
     * @param suffixNoMaskLen
     * @param mask
     * @return
     */
    public static DesensitizedRule buildDesensitizedRule(boolean enable, int prefixNoMaskLen, int suffixNoMaskLen,
            String mask) {
        if (prefixNoMaskLen < 0) {
            prefixNoMaskLen = 0;
        }

        if (suffixNoMaskLen < 0) {
            suffixNoMaskLen = 0;
        }

        DesensitizedRule desensitizedRule = new DesensitizedRule();
        desensitizedRule.setEnable(enable);
        desensitizedRule.setPrefixNoMaskLen(prefixNoMaskLen);
        desensitizedRule.setSuffixNoMaskLen(suffixNoMaskLen);

        if (StringUtils.isNotBlank(mask)) {
            desensitizedRule.setMask(mask);
        }

        return desensitizedRule;
    }

    /**
     * @param desensitized
     * @return
     */
    public static DesensitizedRule buildDesensitizedRule(Desensitized desensitized) {
        DesensitizedRule desensitizedRule = new DesensitizedRule();
        desensitizedRule.setEnable(desensitized.enable());
        desensitizedRule.setPrefixNoMaskLen(desensitized.prefixNoMaskLen());
        desensitizedRule.setSuffixNoMaskLen(desensitized.suffixNoMaskLen());
        desensitizedRule.setMask(desensitized.mask());

        return desensitizedRule;
    }

    /**
     * @param original         原始数据
     * @param desensitizedRule 脱敏规则
     */
    public static String mask(String original, DesensitizedRule desensitizedRule) {
        return DesensitizedUtils.mask(original, desensitizedRule.getPrefixNoMaskLen(),
                desensitizedRule.getSuffixNoMaskLen(), desensitizedRule.getMask());
    }

    /**
     * @param original         原始数据
     * @param prefixNoMaskSize 开始N个字符不脱敏
     * @param suffixNoMaskSize 结尾N个字符不脱敏
     * @param maskFlag         脱敏字符串
     */
    public static String mask(String original, int prefixNoMaskSize, int suffixNoMaskSize, String maskFlag) {
        // 空不需要脱敏
        if (StringUtils.isBlank(original) || original.length() <= prefixNoMaskSize) {
            return original;
        }

        if (prefixNoMaskSize < 0) {
            prefixNoMaskSize = 0;
        }

        if (suffixNoMaskSize < 0) {
            suffixNoMaskSize = 0;
        }

        if (prefixNoMaskSize == 0 && suffixNoMaskSize == 0) {
            return original;
        }

        String left = "";

        if (prefixNoMaskSize >= 1) {
            left = StringUtils.left(original, prefixNoMaskSize);
        }

        String right = "";

        if (suffixNoMaskSize >= 1) {
            right = StringUtils.right(original, suffixNoMaskSize);
        }

        original = StringUtils.rightPad(left, (original.length() - suffixNoMaskSize), maskFlag) + right;

        return original;
    }
}